structure T = Tokens;     

(* Types *)
type pos = int;
type svalue = Tokens.svalue;
type ('a,'b) token = ('a,'b) Tokens.token;
type lexresult= (svalue,pos) token;

type lexarg = string;
type arg = lexarg;

(* Error handling info for yacc *)
val lin = ref 1;
val col = ref 0 ;
val eolpos = ref 0 ;

(* End of file management *)
val eof = fn fileName => T.EOF (!lin,!col) ;

fun len s = List.length(explode s);
val atoi = fn (a,r) => ord(a)-ord(#"0")+10*r;
val atof = fn (a,r) => real(ord(a)-ord(#"0"))+r/10.0;

fun realize l =  let
                        fun split [] l = (l,[])
			   |split (x::xs) l = if(x = #".") then (l,xs)
			                      else (split xs (x::l))
			val (l1,l2) = split l []
                        val num = foldr atoi 0 l1
                        val frac = foldr atof 0.0 l2
                in
                real(num)+frac/10.0
                end;

fun toReal l = if((hd l)= #"~") then (~(realize (tl l))) else (realize l);
fun toInt l = if ((hd l)= #"~") then (~(toInt (tl l))) else (foldl atoi 0 l);
fun chop s = implode(rev(tl(rev(tl(explode s)))));
open DataTypes;
%%
%full
%header (functor BuildAstLexFun(structure Tokens: BuildAst_TOKENS));
%arg (fileName:string);
digit = [0-9];
integer = [1-9][0-9]* | 0 | [~][1-9][0-9]*;
dot = [.];
anat = [0-9]*[1-9];
spaces = [\ \t \n];
alpha = [a-zA-Z];
alphanum = [a-zA-Z0-9_\'];
ws = [\t ];
%%
{ws}+		=> ((col := !col + (len yytext));continue ());
"\n"		=> ((lin := !lin + 1);(col := 0);continue());
";"+		=> ((col := !col + (len yytext));T.NEW(!lin,!col));
"T"		=> ((col := !col + 1);T.TRUE(!lin,!col));
"F"		=> ((col := !col + 1);T.FALSE(!lin,!col));
"A"		=> ((col := !col + 1);T.AND(!lin,!col));
"V"		=> ((col := !col + 1);T.OR(!lin,!col));
"/"		=> ((col := !col + 3);T.DIV(!lin,!col));
"%"		=> ((col := !col + 3);T.MOD(!lin,!col));
"*"		=> ((col := !col + 1);T.MULT(!lin,!col));
"+"		=> ((col := !col + 1);T.PLUS(!lin,!col));
"-"		=> ((col := !col + 1);T.MINUS(!lin,!col));
"!"		=> ((col := !col + 1);T.NEG(!lin,!col));
"("		=> ((col := !col + 1);T.LPAREN(!lin,!col));
")"		=> ((col := !col + 1);T.RPAREN(!lin,!col));
":="		=> ((col := !col + 2);T.ASSIGN(!lin,!col));
"->"		=> ((col := !col + 2);T.IMPL(!lin,!col));
"<-"		=> ((col := !col + 2);T.RIMPL(!lin,!col));
"=="		=> ((col := !col + 2);T.EQUALS(!lin,!col));
"<="		=> ((col := !col + 2);T.LESSEQ(!lin,!col));
">="		=> ((col := !col + 2);T.MOREEQ(!lin,!col));
"<"		=> ((col := !col + 2);T.LESS(!lin,!col));
">"		=> ((col := !col + 2);T.MORE(!lin,!col));
"\""[^"\""]*"\""=> ((col := !col + (len yytext));T.STRING((chop yytext),!lin,!col));
{alpha}{alphanum}* => ((col := !col + (len yytext));T.IDENTIFIER(yytext,!lin,!col));
{integer}{dot}{anat} | {integer}{dot}0 | "0.0" => ((col := !col + (len yytext));T.REAL((toReal (explode yytext)),!lin,!col));
{integer}	=> ((col := !col + (len yytext));T.NUM((toInt (explode yytext)),!lin,!col));
.		=> (continue ());
